@@ -4,7 +4,7 @@
  * Project: GGZ Chess game module
  * Date: 03/01/01
  * Desc: Game main functions
- * $Id: game.c 7748 2006-01-07 22:02:13Z jdorje $
+ * $Id: game.c 8780 2007-01-02 12:15:46Z josef $
  *
  * Copyright (C) 2000 Ismael Orenstein.
  *
@@ -40,6 +40,11 @@
 #include "chess.h"
 #include "game.h"
 #include "ai.h"
+#include "ai-gnuchess.h"
+
+#define AI_UNKNOWN -1
+#define AI_GGZCHESS 0
+#define AI_GNUCHESS 1
 
 /* Game from cgc */
 game_t *game;
@@ -49,10 +54,12 @@ struct chess_info game_info;
  * Use to store the time of the current play */
 struct timeval chronometer;
 /* Savegame file */
-static FILE *savegame;
+static FILE *savegame = NULL;
 /* Moves table */
 /*static char moves[1000][6];
 static int currentmove;*/
+/* Type of AI: -1 = still unknown, 0 = ggzai, 1 = gnuchess */
+static int ai_type = AI_UNKNOWN;
 
 static void game_restart_chronometer(void);
 static int game_read_chronometer(void);
@@ -101,8 +108,10 @@ void game_handle_ggz_state(GGZdMod *ggz,
   const GGZdModState *old_state = data;
   GGZdModState new_state = ggzdmod_get_state(ggz);
 
-  if (*old_state == GGZDMOD_STATE_CREATED)
+  if (*old_state == GGZDMOD_STATE_CREATED) {
+    /* We've arrived in GGZDMOD_STATE_WAITING for the first time */
     game_update(CHESS_EVENT_LAUNCH, NULL);
+  }
 
   if (new_state == GGZDMOD_STATE_DONE);
     game_update(CHESS_EVENT_QUIT, NULL);
@@ -154,6 +163,7 @@ static int game_update(int event_id, const void *data)
   int ret;
   char botmove[6];
   int color, seatnum;
+  const char *aiclass;
 
   switch (event_id) {
     case CHESS_EVENT_LAUNCH:
@@ -170,7 +180,23 @@ static int game_update(int event_id, const void *data)
         color = ((seatnum == 0) ? WHITE : BLACK);
         if (cgc_join_game(game, color) < 0)
           return -1;
-        chess_ai_init((color == BLACK ? C_BLACK : C_WHITE), 2);
+
+        aiclass = ggzdmod_get_bot_class(game_info.ggz, ggzdmod_get_seat(game_info.ggz, seatnum).name);
+        ai_type = AI_GGZCHESS;
+        if (aiclass && !strcmp(aiclass, "gnuchess"))
+          ai_type = AI_GNUCHESS;
+        /* FIXME: fallback to ggzchess, in case no gnuchess is installed? */
+
+		if (ai_type == AI_GGZCHESS) {
+			chess_ai_init((color == BLACK ? C_BLACK : C_WHITE), 2);
+		} else {
+			gnuchess_ai_init((color == BLACK ? C_BLACK : C_WHITE), 2);
+			ret = gnuchess_launch();
+			if(!ret) {
+				ggzdmod_log(game_info.ggz, "error: gnuchess launch failed!");
+				return -1;
+			}
+		}
       }
       break;
     case CHESS_EVENT_JOIN:
@@ -240,7 +266,15 @@ static int game_update(int event_id, const void *data)
       seatnum = ((ggzdmod_get_seat(game_info.ggz, 0).type == GGZ_SEAT_BOT) ? 0 : 1);
       if ((ggzdmod_count_seats(game_info.ggz, GGZ_SEAT_BOT) == 1)
       && (game_info.turn % 2 == seatnum)) {
-        ret = chess_ai_find((seatnum == 0 ? C_WHITE : C_BLACK), &from, &to);
+		if (ai_type == AI_GGZCHESS) {
+        	ret = chess_ai_find((seatnum == 0 ? C_WHITE : C_BLACK), &from, &to);
+		} else {
+        	ret = gnuchess_ai_find((seatnum == 0 ? C_WHITE : C_BLACK), &from, &to);
+			if (!gnuchess_check()) {
+				ggzdmod_log(game_info.ggz, "error: gnuchess has died");
+				return -1;
+			}
+		}
         botmove[0] = from % 8 + 'A';
         botmove[1] = from / 8 + '1';
         botmove[2] = to % 8 + 'A';
@@ -269,9 +303,14 @@ static int game_update(int event_id, const void *data)
       from = ((char*)data)[0] - 65 + (((char*)data)[1] - 49) * 8;
       to = ((char*)data)[2] - 65 + (((char*)data)[3] - 49) * 8;
       ggz_debug(DEBUG_GAME, "** ai: execute %i->%i\n", from, to);
-      ret = chess_ai_move(from, to, 1);
+	  if (ai_type == AI_GGZCHESS) {
+     	 ret = chess_ai_move(from, to, 1);
+	  } else {
+     	 ret = gnuchess_ai_move(from, to, 1);
+	  }
       ggz_debug(DEBUG_GAME, "** ai: executed with result %i\n", ret);
-      chess_ai_output();
+	  if (ai_type == AI_GGZCHESS)
+    	  chess_ai_output();
 
       /* Move was valid */
       if (game_info.clock_type == CHESS_CLOCK_NOCLOCK)
@@ -336,7 +375,15 @@ static int game_update(int event_id, const void *data)
       if ((ggzdmod_count_seats(game_info.ggz, GGZ_SEAT_BOT) == 1)
       && (game_info.turn % 2 == seatnum)) {
         ggz_debug(DEBUG_GAME, "** now move chess bot!\n");
-        ret = chess_ai_find((seatnum == 0 ? C_WHITE : C_BLACK), &from, &to);
+		if (ai_type == AI_GGZCHESS) {
+       		ret = chess_ai_find((seatnum == 0 ? C_WHITE : C_BLACK), &from, &to);
+		} else {
+       		ret = gnuchess_ai_find((seatnum == 0 ? C_WHITE : C_BLACK), &from, &to);
+			if (!gnuchess_check()) {
+				ggzdmod_log(game_info.ggz, "error: gnuchess has died");
+				return -1;
+			}
+		}
         ggz_debug(DEBUG_GAME, "** would move from %i to %i! (valid: %i)\n", from, to, ret);
         botmove[0] = from % 8 + 'A';
         botmove[1] = from / 8 + '1';
@@ -730,6 +777,11 @@ static void game_send_gameover(char code)
   int i;
   char movestr[8], movestr2[8];
 
+  /* Make sure to kill external AI player first */
+  if (ai_type == AI_GNUCHESS) {
+    gnuchess_shutdown();
+  }
+
   /* Report results to GGZ. */
   results[winner] = GGZ_GAME_WIN;
   results[1 - winner] = GGZ_GAME_LOSS;
@@ -819,7 +871,7 @@ static void game_save(char *fmt, ...)
   va_list ap;
 
   if(!savegame) {
-    savegamepath = ggz_strdup(DATADIR "/gamedata/Chess/savegame.pgn.XXXXXX");
+    savegamepath = ggz_strdup(GGZDDATADIR "/gamedata/Chess/savegame.pgn.XXXXXX");
     fd = mkstemp(savegamepath);
     ggz_free(savegamepath);
     if(fd < 0) return;
