@@ -4,7 +4,7 @@
  * Project: GGZCards Client-Common
  * Date: 07/22/2001 (as common.c)
  * Desc: Frontend to GGZCards Client-Common
- * $Id: client.h 4990 2002-10-22 08:32:21Z jdorje $
+ * $Id: client.h 8561 2006-08-31 08:00:24Z jdorje $
  *
  * Copyright (C) 2001-2002 Brent Hendricks.
  *
@@ -28,6 +28,7 @@
 #endif
 
 #include <ggz_common.h>
+#include <ggz_dio.h>
 #ifdef GUI_CLIENT
 #  include <ggzmod.h>
 #endif
@@ -54,6 +55,7 @@ typedef struct seat_t {
 	GGZSeatType status;	/**< ggz seating assignment info */
 	char *name;		/**< player's name */
 	int ggzseat;		/**< GGZ seat number (or -1) */
+	int team;		/**< The seat's team (or -1). */
 
 	/** @brief Player's card on the table.
 	 *
@@ -65,9 +67,10 @@ typedef struct seat_t {
 	 *  need not correspond with what is actually drawn by the GUI.
 	 */
 	card_t table_card;
-	
+	int tricks;
+
 	hand_t hand;		/**< player's hand */
-	
+
 	/** @brief The size of the uncompressed hand.
 	 *
 	 *  The player's "hand" is stored in the hand structure, above.
@@ -79,14 +82,21 @@ typedef struct seat_t {
 	 *  are cards that have already been played.  (The "u" stands
 	 *  for "uncollapsed", I suppose.) */
 	int u_hand_size;
-	
+
+	bool bidding, playing;
+
 	/** @see u_hand_size */
 	struct {
 		bool is_valid;
-		card_t card;		
+		card_t card;
 	} *u_hand;
 } seat_t;
 
+typedef struct {
+	score_data_t score;
+	score_data_t *scores;
+} team_t;
+
 /** @} end of Player group */
 
 
@@ -109,8 +119,17 @@ typedef enum {
 struct ggzcards_game_t {
 	int num_players;	/**< The number of players in the game. */
 	seat_t *players;	/**< Data about each player */
+
+	int num_teams;		/**< The number of teams in the game. */
+	team_t *teams;		/**< Data about each team */
+
 	client_state_t state;	/**< The state the game is in */
 
+	const char *gametype;	/**< String containing the game name. */
+	int hand_num; /**< The number of the current hand (starts at 0) */
+
+	char trump_suit;
+
 	/** @brief The hand we're currently playing from.
 	 *
 	 *  This is the hand we're currently playing from, or -1 if
@@ -118,7 +137,7 @@ struct ggzcards_game_t {
 	 *  is sent to the server, it's not reset to -1 until
 	 *  we get verification.
 	 */
-	int play_hand;		
+	int play_hand;
 };
 
 #ifdef GUI_CLIENT
@@ -162,7 +181,7 @@ void client_quit(void);
 /** This function will return the file descriptor that is used to communicate
  *  with the server.
  */
-int client_get_fd(void);
+GGZDataIO *client_get_dio(void);
 
 /** @} end of Setup */
 
@@ -175,7 +194,7 @@ int client_get_fd(void);
 
 /** Handles an update of the server socket.  This will only happen once, when
  *  we first connect to the server. */
-extern void game_alert_server(int server_socket_fd);
+extern void game_alert_server(GGZDataIO * dio);
 
 /** Handles a newgame request by calling client_send_newgame when
  *  ready (you may wish to ask the user first). */
@@ -187,6 +206,9 @@ extern void game_alert_newgame(cardset_type_t cardset_type);
 /** Alerts the table to the start of a new hand. */
 extern void game_alert_newhand(void);
 
+/** Alerts the table to a change in trump suit. */
+extern void game_alert_trump(void);
+
 /** Handles a gameover message.
  *  @param num_winners The number of players who won the game (0 or more)
  *  @param winners An array with the player numbers of the winners. */
@@ -198,8 +220,8 @@ extern void game_handle_gameover(int num_winners, int *winners);
  *  @param old_status The previous type of the seat.
  *  @param old_name The previous name of the player (seat). */
 extern void game_alert_player(int player,
-                              GGZSeatType old_status,
-                              const char *old_name);
+			      GGZSeatType old_status,
+			      const char *old_name);
 
 /** Alert the table that the number of players has changed.  The
  *  table will probably want to redesign itself.
@@ -209,6 +231,15 @@ extern void game_alert_player(int player,
  *  @note ggzcards.num_players will have been updated. */
 extern void game_alert_num_players(int new, int old);
 
+/** Alert the table that scores data is available.  The table may then
+ *  display or update scores.
+ *  @param hand_num The hand number for which new scores are available. */
+extern void game_alert_scores(int hand_num);
+
+/** Alert the table that tricks data is available.  The table may then
+ *  display or update tricks for each player. */
+extern void game_alert_tricks_count(void);
+
 /** Alerts the table to the maximum hand size.  There will never be more
  *  than this many cards in a hand (unless we send another alert). */
 extern void game_alert_hand_size(int max_hand_size);
@@ -217,21 +248,23 @@ extern void game_alert_hand_size(int max_hand_size);
  *  @param player The player number of the player whose hand has changed. */
 extern void game_display_hand(int player);
 
+/** Called when players status (player.is_bidding and player.is_playing)
+ *  are changed. */
+extern void game_alert_players_status(void);
+
 /** Called to request a bid.  The frontend should call client_send_bid at
   * any point afterwards to send the response.
   * @param possible_bids The number of possible bid choices.
   * @param bid_choices An array of strings, one corresponding to each bid choice. */
 extern void game_get_bid(int possible_bids,
-                         bid_t *bid_choices,
-                         char **bid_texts,
-                         char **bid_descs);
+			 bid_t * bid_choices,
+			 char **bid_texts, char **bid_descs);
 
 /** Called to request a play.  The frontend should call client_send_play
  *  at any point afterwards to send the response.
  *  @param hand The player number of the hand to play from. */
 extern void game_get_play(int play_hand,
-                          int num_valid_cards,
-                          card_t *valid_cards);
+			  int num_valid_cards, card_t * valid_cards);
 
 /** Called when we're informed of a bid.  This data is not tracked
  *  internally and is of little use to the GUI (but would be crucial
@@ -253,7 +286,7 @@ extern void game_alert_badplay(char *err_msg);
  *  @param pos The (former) position of the card in a compressed hand.
  *  @param hand_pos The position of the card in an uncompressed hand. */
 extern void game_alert_play(int player, card_t card,
-                            int pos, int hand_pos);
+			    int pos, int hand_pos);
 
 /** Called when we're informaed of the table cards.  The information
  *  itself resides in the player structures; all the frontend has to
@@ -276,10 +309,10 @@ extern void game_alert_trick(int player);
  *  @return 0 if you are going to handle the option request, -1 if you aren't
  *  @note All parameters are freed after the function returns. */
 extern int game_get_options(int option_cnt,
-                            char **descriptions,
-                            int *choice_cnt,
-                            int *defaults,
-			    char ***option_choices);
+			    char **types,
+			    char **descriptions,
+			    int *choice_cnt,
+			    int *defaults, char ***option_choices);
 
 /** A gui-dependent function called to set a global TEXT message.
  *  This should be defined by the frontend code and is accessed by a
@@ -318,10 +351,8 @@ extern void game_set_player_message(int player, const char *msg);
  *  @param fd The socket from which data may be read.
  *  @param game The type of game (which may or may not be useful).
  *  @param mark The "mark" of the message (which may or may not be useful).
- *  @param size The amount of data ready to be read.
- *  @return The number of bytes read, or negative for error.
- *  @note When in doubt, just use "return 0". */
-extern int game_handle_game_message(int fd, const char *game, int size);
+ *  @note When in doubt, leave function blank. */
+extern void game_handle_game_message(GGZDataIO * dio, const char *game);
 
 /** @} end of Callbacks */
 
@@ -332,35 +363,30 @@ extern int game_handle_game_message(int fd, const char *game, int size);
  *  @{ */
 
 /** Sends the language to the server. */
-int client_send_language(const char *lang);
+void client_send_language(const char *lang);
 
 /** Sends a simple newgame response.
- *  @return 0 on success, -1 on failure.
  *  @see game_get_newgame */
-int client_send_newgame(void);
+void client_send_newgame(void);
 
 /** Sends a bid response.
  *  @param bid The index of the bid chosen.
- *  @return 0 on success, -1 on failure
  *  @see game_get_bid */
-int client_send_bid(int bid);
+void client_send_bid(int bid);
 
 /** Sends an options response.
  *  @param option_cnt The number of options.
  *  @param options The choice made for each option.
- *  @return 0 on success, -1 on failure
  *  @see game_get_options */
-int client_send_options(int option_cnt, int *options);
+void client_send_options(int option_cnt, int *options);
 
 /** Sends a play response.
  *  @param card The card chosen to be played.
- *  @return 0 on success, -1 on failure
  *  @see game_get_play, game_alert_badplay */
-int client_send_play(card_t card);
+void client_send_play(card_t card);
 
-/** Sends a request for a sync.
- *  @return 0 on success, -1 on failure. */
-int client_send_sync_request(void);
+/** Sends a request for a sync. */
+void client_send_sync_request(void);
 
 /** @} end of Responses */
 
