@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 06/20/2001
  * Desc: Functions and data common to all games
- * $Id: common.h 6892 2005-01-25 04:09:21Z jdorje $
+ * $Id: common.h 8561 2006-08-31 08:00:24Z jdorje $
  *
  * This file contains code that controls the flow of a general
  * trick-taking game.  Game states, event handling, etc. are all
@@ -33,7 +33,7 @@
 
 #include <assert.h>
 
-#include <ggz.h>		/* libggz */
+#include <ggz.h>	/* libggz */
 #include "ggzdmod.h"
 
 #include "ai.h"
@@ -43,13 +43,13 @@
 #include "types.h"
 
 /* Debugging/logging types. */
-#define DBG_BID "bid"		/* bid system messages */
-#define DBG_PLAY "play"		/* play system messages */
-#define DBG_AI "ai"		/* AI messages */
-#define DBG_GAME "game"		/* game-specific messages */
-#define DBG_NET "net"		/* net-related messages */
+#define DBG_BID "bid"	/* bid system messages */
+#define DBG_PLAY "play"	/* play system messages */
+#define DBG_AI "ai"	/* AI messages */
+#define DBG_GAME "game"	/* game-specific messages */
+#define DBG_NET "net"	/* net-related messages */
 #define DBG_CLIENT "client"	/* client problems */
-#define DBG_MISC "misc"		/* everything else... */
+#define DBG_MISC "misc"	/* everything else... */
 
 /* GGZCards server game states */
 typedef enum {
@@ -89,10 +89,15 @@ typedef struct {
 	bool must_overtrump;	/**< if this is set, then a player must trump/overtrump if possible (i.e. La Pocha) */
 	bool must_break_trump;	/**< if this is set, then trump can't be lead until it's been broken (i.e. Spades) */
 #define MAX_TARGET_SCORE 1000000
-	int target_score;	/**< after someone reaches this score, the game is over (0=unused) */
+	bool ties_allowed;	/**< set to true if ties are allowed; otherwise the game won't end automatically when one of the conditions is reached equally by both teams. */
+	int target_score;	/**< after someone reaches this score, the game is over */
+#define MAX_HANDS 1000000
+	int max_hands; /**< after this many hands have been played, the game is over. */
+	int forfeit_score;	/**< If someone's score drops this low, the game is over (0=unused) */
 	bool last_trick;	/**< should the last trick be sent to all the players? */
 	bool last_hand;		/**< should the last hand be sent to all the players? */
 	bool cumulative_scores;	/**< should the cumulative score be sent to all the players? */
+	int num_extra_scores;
 	bool bid_history;	/**< should a complete history of the hand's bidding be shown? */
 	bool rated;		/**< should we track player stats? */
 
@@ -137,13 +142,16 @@ typedef struct {
 	int num_players;	/**< the number of players in the game */
 	int player_count;	/**< the number of human players who have joined, in total */
 	struct game_player_t *players;	/**< data for each player, allocated in game_init */
+	struct game_spectator_t *spectators; /**< data for each spectator. */
+	int spectator_count; /**< Size of the spectators array. */
 
 	/* table data: seats */
 	int num_seats;	/**< the number of "seats" in the table (which includes fake non-players */
 	struct game_seat_t *seats;	/**< data for each seat, allocated in game_init */
-	
+
 	/* table data: teams */
 	int num_teams;
+	struct game_team_t *teams;
 
 	/* Global messages */
 	global_message_list_t *message_head;	/**< global message list head */
@@ -162,13 +170,13 @@ void set_game_state(server_state_t state);
 /* these are GGZ communication events that we must handle */
 void handle_ggz_state_event(GGZdMod * ggz,
 			    GGZdModEvent event, const void *data);
-void handle_ggz_seat_event(GGZdMod *ggz, 
+void handle_ggz_seat_event(GGZdMod * ggz,
 			   GGZdModEvent event, const void *data);
-void handle_ggz_spectator_seat_event(GGZdMod *ggz,
+void handle_ggz_spectator_seat_event(GGZdMod * ggz,
 				     GGZdModEvent event, const void *data);
 void handle_ggz_player_data_event(GGZdMod * ggz,
 				  GGZdModEvent event, const void *data);
-void handle_ggz_spectator_data_event(GGZdMod *ggz,
+void handle_ggz_spectator_data_event(GGZdMod * ggz,
 				     GGZdModEvent event, const void *data);
 
 /* Handle a player/spectator data event. */
@@ -182,19 +190,19 @@ void handle_neterror_event(player_t p);
 
 /* Handlers for general client messages.  Other handlers may be in
    their specific files. */
-void handle_client_language(player_t p, const char* lang);
+void handle_client_language(player_t p, const char *lang);
 void handle_client_newgame(player_t p);
 void handle_client_sync(player_t p);
 
 /* General high-level functions */
-void init_ggzcards(GGZdMod * ggz, game_data_t *game_data);
+void init_ggzcards(GGZdMod * ggz, game_data_t * game_data);
 bool try_to_start_game(void);
 void init_game(void);
 void next_move(void);		/* make the next move, if all seats are full */
 void send_sync(player_t p);
 void broadcast_sync(void);
 void send_hand(const player_t p, const seat_t s,
-               bool show_fronts, bool show_backs);
+	       bool show_fronts, bool show_backs);
 
 /* Seat manipulation functions */
 void set_num_seats(int num_seats);
@@ -211,13 +219,15 @@ extern game_t game;
 const char *get_seat_name(seat_t s);
 GGZSeatType get_seat_status(seat_t s);
 
-const char* get_player_name(player_t p);
+const char *get_player_name(player_t p);
 GGZSeatType get_player_status(player_t p);
-int get_player_socket(int p);
+GGZDataIO *get_player_dio(int p);
 
 bool seats_full(void);
 bool seats_empty(void);
 
+void set_trump_suit(char new_trump_suit);
+
 #define SPECTATOR_TO_PLAYER(spectator) (-1 - (spectator))
 #define PLAYER_TO_SPECTATOR(player) (-1 - (player))
 
@@ -244,13 +254,19 @@ void fatal_error(const char *message);
 #define players_iterate(p)                                  \
 {                                                           \
 	player_t p;                                         \
-		for (p = 0; p < game.num_players; p++)
+	for (p = 0; p < game.num_players; p++)
 
 #define players_iterate_end }
 
+#define teams_iterate(t)				    \
+{							    \
+	team_t t;					    \
+	for (t = 0; t < game.num_teams; t++)
+
+#define teams_iterate_end }
+
 #define IS_REAL_PLAYER(p) ((p) >= 0)
 
 #define IS_SPECTATOR(p) ((p) < 0)
 
 #endif /* __COMMON_H__ */
-
