@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 06/29/2000
  * Desc: default game functions
- * $Id: game.c 5017 2002-10-23 22:19:46Z jdorje $
+ * $Id: game.c 8565 2006-08-31 22:42:10Z jdorje $
  *
  * This file was originally taken from La Pocha by Rich Gade.  It now
  * contains the default game functions; that is, the set of game functions
@@ -44,6 +44,7 @@
 #include "net.h"
 #include "options.h"
 #include "play.h"
+#include "score.h"
 #include "team.h"
 
 static void bad_game(char *func)
@@ -79,14 +80,14 @@ void game_init_game(void)
    as-is for a game that has no options. */
 void game_get_options(void)
 {
-	add_option("open_hands",
+	add_option("General Options", "open_hands",
 	           "Select this option to have all hands visible to everyone.",
 	           1, 0, "Play with open hands");
-	add_option("blind_spectators",
+	add_option("General Options", "blind_spectators",
 		   "Select this option to hide everybody's hand from all "
 		   "spectators.",
-		   1, 0, "Hide hands from spectators");
-	add_option("rated_game",
+		   1, 1, "Hide hands from spectators");
+	add_option("General Options", "rated_game",
 	           "Select this option to have the game rated.",
 	           1, 1, "Play a rated game");
 }
@@ -165,6 +166,13 @@ void game_set_player_message(player_t p)
 }
 
 
+/* Called to send a sync to a given player (including spectators). */
+void game_sync_player(player_t p)
+{
+	/* Nothing */
+}
+
+
 /* Places text for the bid into the buffer.  Returns the length of the text. */
 int game_get_bid_text(char *buf, size_t buf_len, bid_t bid)
 {
@@ -394,10 +402,7 @@ void game_end_trick(void)
 	game.players[hi_player].tricks++;
 	game.leader = game.winner = hi_player;
 
-	if (game.players[hi_player].team >= 0)
-		map_func_to_team(game.players[hi_player].team, set_player_message);
-	else
-		set_player_message(hi_player);
+	map_func_to_team(game.players[hi_player].team, set_player_message);
 }
 
 
@@ -413,12 +418,11 @@ void game_end_hand(void)
    not necessary yet anyway. */
 void game_start_game(void)
 {
-	player_t p;
-
 	/* TODO: initialize the game; right now we just assume everything's
 	   zero which won't be true the second time around. */
-	for (p = 0; p < game.num_players; p++)
-		game.players[p].score = 0;
+	teams_iterate(t) {
+		clear_score(t);
+	} teams_iterate_end;
 }
 
 
@@ -426,12 +430,62 @@ void game_start_game(void)
    Return 1 for gameover, 0 otherwise. */
 bool game_test_for_gameover(void)
 {
-	player_t p;
+	int max_score, max_score_count = 0;
+	int min_score, min_score_count = 0;
+	team_t max_score_team = -1, min_score_team = -1;
+
+	/* We have to check for ties (not allowed in most games), so this
+	   code is a good bit more complicated.  For instance in spades if
+	   both teams go over the target score and the game is tied, another
+	   hand is played. */
+	teams_iterate(t) {
+		/* in the default case, it's just a race toward a
+		   target score */
+		int score = get_team_score(t);
+
+		if (game.target_score != 0 && score >= game.target_score) {
+			if (max_score_count == 0 || score > max_score) {
+				max_score = score;
+				max_score_count = 1;
+				max_score_team = t;
+			} else if (score < max_score) {
+				/* nothing */
+			} else if (score == max_score) {
+				/* check for ties */
+				max_score_count++;
+			}
+		}
+
+		/* It is possible to set a low score that will cause a
+		   forfeit if reached. */
+		if (game.forfeit_score != 0 && score <= game.forfeit_score) {
+			if (min_score_count == 0 || score < min_score) {
+				min_score = score;
+				min_score_count = 1;
+				min_score_team = t;
+			} else if (score > min_score) {
+				/* nothing */
+			} else if (score == min_score) {
+				/* check for ties */
+				min_score_count++;
+			}
+		}
+	} teams_iterate_end;
+	if (max_score_count > 0
+	    && (game.ties_allowed || max_score_count == 1)) {
+		return TRUE;
+	}
+	if (min_score_count > 0
+	    && (game.ties_allowed || min_score_count == 1)) {
+		return TRUE;
+	}
+
+	/* Sometimes there is a limit on the maximum number of hands to be
+	   played. */
+	if (game.max_hands != 0
+	    && game.hand_num >= game.max_hands)
+	  return TRUE;
 
-	/* in the default case, it's just a race toward a target score */
-	for (p = 0; p < game.num_players; p++)
-		if (game.players[p].score >= game.target_score)
-			return TRUE;
 	return FALSE;
 }
 
@@ -447,23 +501,24 @@ card_t game_map_card(card_t c)
    determines who has won and calls send_gameover. */
 void game_handle_gameover(void)
 {
-	player_t p;
 	int hi_score = -9999;
 	player_t winners[game.num_players];
 	int winner_cnt = 0;
 
 	/* in the default case, just take the highest score(s) this should
 	   automatically handle the case of teams! */
-	for (p = 0; p < game.num_players; p++) {
-		if (game.players[p].score > hi_score) {
+	players_iterate(p) {
+		int score = get_player_score(p);
+
+		if (score > hi_score) {
 			winner_cnt = 1;
 			winners[0] = p;
-			hi_score = game.players[p].score;
-		} else if (game.players[p].score == hi_score) {
+			hi_score = score;
+		} else if (score == hi_score) {
 			winners[winner_cnt] = p;
 			winner_cnt++;
 		}
-	}
+	} players_iterate_end;
 
 	handle_gameover_event(winner_cnt, winners);
 }
@@ -473,15 +528,16 @@ void game_handle_gameover(void)
    function is used for the automatic sorting of hands by cards_sort_hand. */
 int game_compare_cards(card_t card1, card_t card2)
 {
-	if (card1.suit < card2.suit)
-		return -1;
-	if (card1.suit > card2.suit)
-		return 1;
-	if (card1.face < card2.face)
-		return -1;
-	if (card1.face > card2.face)
-		return 1;
-	return 0;		/* ignore decks for now */
+	if (card1.suit != card2.suit) {
+		int order[] = {2, 1, 3, 4};
+
+		/* HACK: Suits aren't put in order; instead they alternate
+		   color. */
+		assert(card1.suit >= 0 && card1.suit < 4);
+		assert(card2.suit >= 0 && card2.suit < 4);
+		return order[(int)card1.suit] - order[(int)card2.suit];
+	}
+	return card1.face - card2.face;		/* ignore decks for now */
 }
 
 
@@ -492,7 +548,8 @@ void game_send_hand(player_t p, seat_t s)
 {
 	/* in most cases, we want to reveal the hand only to the player who
 	   owns it. */
-	bool show_fronts = (game.players[p].seat == s);
+	bool show_fronts = (p >= 0 && game.players[p].seat == s);
 	bool show_backs = TRUE;
+
 	send_hand(p, s, show_fronts, show_backs);
 }
