@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 06/20/2001
  * Desc: Game-independent game network functions
- * $Id: net.c 8561 2006-08-31 08:00:24Z jdorje $
+ * $Id: net.c 9111 2007-05-13 02:28:51Z jdorje $
  *
  * This file contains code that controls the flow of a general
  * trick-taking game.  Game states, event handling, etc. are all
@@ -536,8 +536,9 @@ void net_broadcast_newgame(void)
 static void net_send_newhand(player_t p)
 {
 	GGZDataIO *dio = get_player_dio(p);
-	seat_t dlr =
-	    game.dealer >= 0 ? game.players[game.dealer].seat : -1;
+	seat_t dlr = (game.dealer >= 0
+		      ? CONVERT_SEAT(game.players[game.dealer].seat, p)
+		      : -1);
 
 	ggz_dio_packet_start(dio);
 	write_opcode(dio, MSG_NEWHAND);
