@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 02/27/2002
  * Desc: Functions and data for tracking teams
- * $Id: team.c 3574 2002-03-16 15:56:43Z jdorje $
+ * $Id: team.c 8457 2006-08-02 06:15:44Z jdorje $
  *
  * Copyright (C) 2001-2002 Brent Hendricks.
  *
@@ -28,6 +28,7 @@
 #endif
 
 #include <assert.h>
+#include <stdlib.h>
 
 #include "common.h"
 #include "team.h"
@@ -35,6 +36,10 @@
 void set_num_teams(int num_teams)
 {
 	game.num_teams = num_teams;
+	game.teams = ggz_malloc(game.num_teams * sizeof(*game.teams));
+	teams_iterate(t) {
+		game.teams[t].scores = NULL;
+	} teams_iterate_end;
 }
 
 void assign_team(team_t t, player_t p)
@@ -67,14 +72,15 @@ int get_team_tricks(team_t t)
 	return tricks;
 }
 
-int get_team_score(team_t t)
+team_t get_player_team(player_t p)
 {
-	player_t p;
-	
-	for (p = 0; p < game.num_players; p++)
-		if (game.players[p].team == t)
-			return game.players[p].score;
-			
-	assert(FALSE);
-	return 0;
+	/* This applies to spectators or empty seats. */
+	if (p < 0) return NO_TEAM;
+
+	return game.players[p].team;
+}
+
+team_t get_seat_team(seat_t s)
+{
+	return get_player_team(SEAT_TO_PLAYER(s));
 }
