@@ -4,7 +4,7 @@
  * Project: GGZCards Server
  * Date: 06/29/2000
  * Desc: Special types for GGZCards game data
- * $Id: types.h 4025 2002-04-20 09:10:07Z jdorje $
+ * $Id: types.h 8465 2006-08-03 07:29:12Z jdorje $
  *
  * These are a few random type definitions used all over the place.
  *
@@ -30,6 +30,7 @@
 
 #include <sys/types.h>
 
+#include "ggz_dio.h"
 #include "shared.h"
 
 #include "deck.h"
@@ -37,7 +38,7 @@
 
 /* This struct holds all data about a player's bidding status. */
 typedef struct {
-	bool is_bidding;		/* is this player currently bidding? Are we
+	bool is_bidding;	/* is this player currently bidding? Are we
 				   waiting for a bid from them? */
 
 	/* FIXME: should this use libggz's list structure? */
@@ -65,7 +66,6 @@ struct game_seat_t {
 };
 
 struct game_player_t {
-	int score;
 	int bid_count;		/* the number of times this player has bid
 				   this hand */
 	int age;		/* how long the player has been at the table; 
@@ -78,16 +78,17 @@ struct game_player_t {
 	seat_t seat;		/* the number of the player's seat. */
 	team_t team;		/* The player's team, or -1 for none. */
 	bool ready;
-	
+
+	GGZDataIO *dio;
+
 	/* Used for tracking AI processees. */
-	int fd;
 #ifdef DEBUG
 	int err_fd;
-#endif /* DEBUG */
+#endif				/* DEBUG */
 	pid_t pid;
 
 	bid_data_t bid_data;	/* bidding information for this player */
-	
+
 	bool is_playing;	/* is this player currently playing? */
 	/* Note: In the case of bridge, game.next_play will go
 	   around the table (0, 1, 2, 3) for each play.  However, when the
@@ -96,6 +97,15 @@ struct game_player_t {
 	seat_t play_seat;	/* what seat are we playing from? */
 };
 
+struct game_team_t {
+	score_data_t score;
+	score_data_t *scores;	/* Scores from every hand. */
+};
+
+struct game_spectator_t {
+	GGZDataIO *dio;
+};
+
 /* Used in message-sending in message.c. */
 typedef struct global_message_list_t global_message_list_t;
 
