@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 5/10/00
  * Desc: Functions for handling/manipulating GGZ chat/messaging
- * $Id: chat.c 5928 2004-02-15 02:43:16Z jdorje $
+ * $Id: chat.c 8905 2007-01-13 12:18:47Z oojah $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -66,16 +66,21 @@ GGZClientReqError chat_room_enqueue(int room, GGZChatType type,
 				    GGZPlayer *sender, const char *msg)
 {
 	GGZChatEventData *data;
-	int i, rooms;
+	int i, roomnum;
 
 	/* A message to room -1 announces to all rooms */
 	if(room == -1) {
 		GGZClientReqError status = E_OK;
-		if(perms_check(sender, PERMS_CHAT_ANNOUNCE) != PERMS_ALLOW)
+		if (!perms_check(sender, GGZ_PERM_CHAT_ANNOUNCE)) {
 			return E_NO_PERMISSION;
-		rooms = room_get_num_rooms();
-		for(i=0; i<rooms; i++) {
+		}
+		roomnum = room_get_num_rooms();
+		for(i = 0; i < roomnum; i++) {
 			GGZClientReqError result;
+
+			if (room_is_removed(i))
+				continue;
+
 			result = chat_room_enqueue(i, type, sender, msg);
 			if (result != E_OK)
 				status = result;
@@ -86,6 +91,16 @@ GGZClientReqError chat_room_enqueue(int room, GGZChatType type,
 	/* Pack up chat message */
 	data = chat_pack(type, sender->name, msg);
 
+	if (sender->gagged) {
+		/* Gagged players only chat to themselves */
+		type = GGZ_CHAT_PERSONAL;
+		if (event_player_enqueue(sender->name, chat_event_callback,
+			sizeof(*data), data, chat_free) != GGZ_OK) {
+			return E_BAD_XML;
+		} else
+			return E_OK;
+	}
+
 	if (event_room_enqueue(room, chat_event_callback,
 			       sizeof(*data), data, chat_free) != GGZ_OK)
 		return E_BAD_XML; /* internal server error? */
@@ -130,21 +145,15 @@ GGZClientReqError chat_player_enqueue(const char* receiver, GGZChatType type,
 {
 	GGZPlayer* rcvr;
 	GGZChatEventData *data;
-	bool at_table;
+	bool sender_at_table, rcvr_at_table;
+	bool sender_has_perm, rcvr_has_perm;
+	bool sender_gagged, rcvr_has_hostperm;
+	bool sender_is_rcvr;
 
 	if (receiver == NULL) {
 		return E_BAD_OPTIONS;
 	}
 
-	pthread_rwlock_rdlock(&sender->lock);
-	at_table = (sender->table != -1);
-	pthread_rwlock_unlock(&sender->lock);	
-
-	/* Don't allow personal chat from a player at a table */
-	if (at_table){
-		return E_AT_TABLE;
-	}
-
 	/* Find target player.  Returns with player write-locked */
 	rcvr = hash_player_lookup(receiver);
 
@@ -152,21 +161,66 @@ GGZClientReqError chat_player_enqueue(const char* receiver, GGZChatType type,
 		return E_USR_LOOKUP;
 	}
 
-	/* Don't allow personal chat to a player at a table */
-	at_table = (rcvr->table != -1);
+	rcvr_has_hostperm = ggz_perms_is_host(rcvr->perms);
+
+	/* Get information about the sender */
+	sender_is_rcvr = (sender == rcvr);
+	if(!sender_is_rcvr)
+		pthread_rwlock_rdlock(&sender->lock);
+	sender_at_table = (sender->table != -1);
+	sender_gagged = sender->gagged;
+	if(!sender_is_rcvr)
+		pthread_rwlock_unlock(&sender->lock);	
+
+	/* Don't allow personal chat to a player at a table. See below*/
+	rcvr_at_table = (rcvr->table != -1);
 
 	/* FIXME: we release the lock only to acquire it again in
 	   event_player_enqueue.  This is inefficient. */
 	pthread_rwlock_unlock(&rcvr->lock);
 
-	if (at_table) {
+	/* Check for gagging */
+	if (sender_gagged) {
+		/* Gagged players may only send private messages to hosts/admins... */
+		if (!rcvr_has_hostperm) {
+			/* ... and to themselves of course */
+			if (!sender_is_rcvr) {
+				/* We don't let them find out that they've been gagged. */
+				return E_OK;
+			}
+		}
+	}
+
+	/* Truth table for whether private messages are allowed.
+	 * Perm = has GGZ_PERM_TABLE_PRIVMSG
+	 * NoPerm = doesn't have GGZ_PERM_TABLE_MSG
+	 * At = At a table
+	 * NotAt = Not at a table
+	 *
+	 *               rcvr\sender Perm,At Perm,NotAt NoPerm,NotAt NoPerm,At
+	 *            Perm,At          Y          Y           Y          Y
+	 *            Perm,NotAt       Y          Y           Y          Y
+	 *            NoPerm,NotAt     Y          Y           Y          N
+	 *            NoPerm,At        Y          Y           N          N
+	 *
+	 * This basically means that people with GGZ_PERM_TABLE_PRIVMSG can
+	 * privmsg with anybody and have them privmsg back as well.
+	 * Without being able to privmsg to/from normal players at a table
+	 * it would make this permission pretty useless.
+	 */
+
+	sender_has_perm = perms_check(sender, GGZ_PERM_TABLE_PRIVMSG);
+	rcvr_has_perm = perms_check(rcvr, GGZ_PERM_TABLE_PRIVMSG);
+
+	if ( (!sender_has_perm && !rcvr_has_perm && rcvr_at_table) ||
+					(!rcvr_has_perm && !sender_has_perm && sender_at_table)){
 		return E_AT_TABLE;
 	}
 
 	/* Pack up chat message.  Sender->name won't change so no lock
 	 * is needed. */
 	data = chat_pack(type, sender->name, msg);
-	
+
 	/* Queue chat event for individual player */
 	if (event_player_enqueue(receiver, chat_event_callback,
 				 sizeof(*data), data, chat_free) != GGZ_OK) {
@@ -272,6 +326,7 @@ static GGZEventFuncReturn chat_table_event_callback(void *target, size_t size,
 
 /* This is more or less a temporary hack for show_server_info() - room.c */
 /* Although this could be a useful function for other uses. */
+/* If player name is NULL, message is announced to all rooms */
 GGZReturn chat_server_2_player(const char *name, const char *msg)
 {
 	if (name) {
@@ -284,12 +339,15 @@ GGZReturn chat_server_2_player(const char *name, const char *msg)
 		return event_player_enqueue(name, chat_event_callback,
 					    sizeof(*data), data, chat_free);
 	} else {
-		int rooms = room_get_num_rooms(), i;
+		int roomnum = room_get_num_rooms(), i;
 		GGZReturn status = GGZ_OK;
 
-		for (i = 0; i < rooms; i++) {
+		for (i = 0; i < roomnum; i++) {
 			GGZChatEventData *data;
 
+			if (room_is_removed(i))
+				continue;
+
 			data = chat_pack(GGZ_CHAT_ANNOUNCE, "[Server]", msg);
 
 			if (event_room_enqueue(i, chat_event_callback,
