@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/11/99
  * Desc: Control/Port-listener part of server
- * $Id: control.c 8779 2007-01-02 12:14:04Z josef $
+ * $Id: control.c 8980 2007-02-08 00:38:01Z jdorje $
  *
  * Copyright (C) 1999 Brent Hendricks.
  *
@@ -559,7 +559,6 @@ int main(int argc, char *argv[])
 	struct sockaddr_in addr;
 	fd_set active_fd_set, read_fd_set;
 	struct timeval tv, *tvp;
-	int seconds;
 
 	logfile_preinitialize();
 
@@ -671,17 +670,13 @@ int main(int argc, char *argv[])
 		}
 
 		read_fd_set = active_fd_set;
-		seconds = log_next_update_sec();
-		if(seconds == 0)
-		{
-			tvp = NULL;
-		}
-		else
-		{
-			tv.tv_sec = seconds;
-			tv.tv_usec = 0;
-			tvp = &tv;
-		}
+
+		/* May be 0 seconds if the update interval has passed -
+		   in this case the select will likely terminate immediately
+		   and the update will be done. */
+		tv = ggztime_to_timeval(log_next_update_sec());
+		tvp = &tv;
+
 		status = select((select_max + 1), &read_fd_set, NULL, NULL, tvp);
 
 		if (status < 0) {
