@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 06/11/2000
  * Desc: Front-end functions to handle database manipulation
- * $Id: ggzdb.c 7424 2005-08-15 09:00:27Z josef $
+ * $Id: ggzdb.c 8504 2006-08-08 09:07:31Z josef $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -92,6 +92,7 @@ int ggzdb_init(void)
 	}
 
 	/* Call backend's initialization */
+	connection.type = opt.dbtype;
 	connection.datadir = opt.data_dir;
 	connection.host = opt.dbhost;
 	connection.database = opt.dbname;
@@ -119,7 +120,7 @@ GGZDBResult ggzdb_player_add(ggzdbPlayerEntry *pe)
 	GGZDBResult rc = GGZDB_NO_ERROR;
 	char orig[MAX_USER_NAME_LEN + 1];
 	hash_t hash;
-	char *password64;
+	char *password_enc;
 	char *origpassword = NULL;
 
 	if(!opt.dbhashing) return GGZDB_ERR_DB;
@@ -138,13 +139,21 @@ GGZDBResult ggzdb_player_add(ggzdbPlayerEntry *pe)
 		|| (!strcmp(opt.dbhashing, "ripemd160")))
 		{
 			hash = ggz_hash_create(opt.dbhashing, pe->password);
-			password64 = ggz_base64_encode(hash.hash, hash.hashlen);
-			if(password64)
-			{
-				free(hash.hash);
-				origpassword = strdup(pe->password);
-				snprintf(pe->password, sizeof(pe->password), "%s", password64);
+			if(!strcmp(opt.dbhashencoding, "base16")){
+				password_enc = ggz_base16_encode(hash.hash, hash.hashlen);
+			}else{
+				password_enc = ggz_base64_encode(hash.hash, hash.hashlen);
 			}
+			if(hash.hash)
+				ggz_free(hash.hash);
+
+			if(password_enc)
+			{
+				origpassword = ggz_strdup(pe->password);
+				snprintf(pe->password, sizeof(pe->password), "%s", password_enc);
+				ggz_free(password_enc);
+			}
+			// FIXME - shouldn't failing to encrypt the password be an error?
 		}
 		rc = _ggzdb_player_add(pe);
 	}
@@ -154,7 +163,7 @@ GGZDBResult ggzdb_player_add(ggzdbPlayerEntry *pe)
 	if(origpassword)
 	{
 		snprintf(pe->password, sizeof(pe->password), "%s", origpassword);
-		free(origpassword);
+		ggz_free(origpassword);
 	}
 	
 	_ggzdb_exit();
@@ -373,7 +382,7 @@ static void ggzdb_player_lowercase(ggzdbPlayerEntry *pe, char *buf)
 int ggzdb_compare_password(const char *input, const char *password)
 {
 	hash_t hash;
-	char *password64;
+	char *password_enc;
 	int ret = 0;
 
 	if(!opt.dbhashing) return -1;
@@ -389,11 +398,17 @@ int ggzdb_compare_password(const char *input, const char *password)
 	|| (!strcmp(opt.dbhashing, "ripemd160")))
 	{
 		hash = ggz_hash_create(opt.dbhashing, input);
-		password64 = ggz_base64_encode(hash.hash, hash.hashlen);
-		if(!password64) return -1;
-		if(!strcmp(password64, password)) ret = 1;
-		free(hash.hash);
-		free(password64);
+		if(!strcmp(opt.dbhashencoding, "base16")){
+			password_enc = ggz_base16_encode(hash.hash, hash.hashlen);
+		}else{
+			password_enc = ggz_base64_encode(hash.hash, hash.hashlen);
+		}
+		if(hash.hash)
+			ggz_free(hash.hash);
+
+		if(!password_enc) return -1;
+		if(!strcmp(password_enc, password)) ret = 1;
+		ggz_free(password_enc);
 		return ret;
 	}
 
@@ -402,3 +417,79 @@ int ggzdb_compare_password(const char *input, const char *password)
 	return -1;
 }
 
+/* Helper function, might go into libggz*/
+char *_ggz_sql_escape(const char *str)
+{
+	char *newstr, *q;
+	const char *p;
+	size_t len = 0;
+
+	if(str == NULL)
+		return NULL;
+
+	len = strlen(str);
+
+	for(p = str; *p != '\0'; p++) {
+		if(*p == '\'') {
+			len += 1;
+		}
+	}
+
+	if(len == strlen(str))
+		return ggz_strdup(str);
+
+	newstr = ggz_malloc(len + 1);
+	q = newstr;
+
+	for(p = str; *p != '\0'; p++) {
+		if(*p == '\'') {
+			*q++ = '\\';
+			*q = *p;
+		} else {
+			*q = *p;
+		}
+		q++;
+	}
+	*q = '\0';
+
+	return newstr;
+}
+
+/* Helper function, might go into libggz*/
+/*char *_ggz_sql_unescape(const char *str)
+{
+	char *new, *q;
+	const char *p;
+	size_t len = 0;
+
+	if(str == NULL)
+		return NULL;
+
+	len = strlen(str);
+
+	for(p = str; *p != '\0'; p++) {
+		if(!strncmp(p, "\\'", 2)) {
+			p += 1;
+		}
+		len++;
+	}
+
+	if(len == strlen(str))
+		return ggz_strdup(str);
+
+	q = new = ggz_malloc(len + 1);
+	for(p = str; *p != '\0'; p++) {
+		if(!strncmp(p, "\\'", 2)) {
+			*q = '\'';
+			q++;
+			p += 1;
+		} else {
+			*q = *p;
+			q++;
+		}
+	}
+	*q = '\0';
+
+	return new;
+}*/
+
