@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 11/10/2000
  * Desc: Back-end functions for handling the db4 sytle database
- * $Id: ggzdb_db4.c 7424 2005-08-15 09:00:27Z josef $
+ * $Id: ggzdb_db4.c 8460 2006-08-02 14:29:54Z josef $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -32,11 +32,7 @@
 #include <string.h>
 #include <sys/types.h>
 
-#ifdef DB4_IN_DIR
-#  include <db4/db.h>
-#else
-#  include <db.h>
-#endif
+#include <db.h>
 
 #include "err_func.h"
 #include "ggzd.h"
@@ -82,6 +78,11 @@ void _ggzdb_close(void)
 		db_p = NULL;
 	}
 
+	if(db_s) {
+		db_s->close(db_s, 0);
+		db_s = NULL;
+	}
+
 	db_e->close(db_e, 0);
 	db_e = NULL;
 }
