@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 02.05.2002
  * Desc: Back-end functions for handling the postgresql style database
- * $Id: ggzdb_pgsql.c 7900 2006-03-13 12:34:50Z josef $
+ * $Id: ggzdb_pgsql.c 8374 2006-07-20 12:44:03Z josef $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -162,82 +162,6 @@ static void releaseconnection(PGconn *conn)
 	pthread_mutex_unlock(&mutex);
 }
 
-/* Helper function, might go into libggz*/
-char *_ggz_sql_escape(const char *str)
-{
-	char *new, *q;
-	const char *p;
-	size_t len = 0;
-
-	if(str == NULL)
-		return NULL;
-
-	len = strlen(str);
-
-	for(p = str; *p != '\0'; p++) {
-		if(*p == '\'') {
-			len += 1;
-		}
-	}
-
-	if(len == strlen(str))
-		return ggz_strdup(str);
-
-	q = new = ggz_malloc(len + 1);
-	for(p = str; *p != '\0'; p++) {
-		if(*p == '\'') {
-			*q++ = '\\';
-			*q = *p;
-			*q++;
-		} else {
-			*q = *p;
-			q++;
-		}
-	}
-	*q = '\0';
-
-	return new;
-}
-
-/* Helper function, might go into libggz*/
-/*char *_ggz_sql_unescape(const char *str)
-{
-	char *new, *q;
-	const char *p;
-	size_t len = 0;
-
-	if(str == NULL)
-		return NULL;
-
-	len = strlen(str);
-
-	for(p = str; *p != '\0'; p++) {
-		if(!strncmp(p, "\\'", 2)) {
-			p += 1;
-		}
-		len++;
-	}
-
-	if(len == strlen(str))
-		return ggz_strdup(str);
-
-	q = new = ggz_malloc(len + 1);
-	for(p = str; *p != '\0'; p++) {
-		if(!strncmp(p, "\\'", 2)) {
-			*q = '\'';
-			q++;
-			p += 1;
-		} else {
-			*q = *p;
-			q++;
-		}
-	}
-	*q = '\0';
-
-	return new;
-}*/
-
-
 /* Exported functions */
 
 /* Function to initialize the pgsql database system */
@@ -284,11 +208,12 @@ GGZReturn _ggzdb_init(ggzdbConnection connection, int set_standalone)
 	}
 
 	/* Initialize the database if needed */
+	/* FIXME: this should be read from external schema file with copy-in */
 	if(init)
 	{
 		snprintf(query, sizeof(query), "CREATE TABLE users "
 			"(id serial, handle varchar(256), password varchar(256), name varchar(256), email varchar(256), "
-			"lastlogin int8, permissions int8)");
+			"lastlogin int8, permissions int8, firstlogin int8)");
 
 		res = PQexec(conn, query);
 
@@ -409,10 +334,10 @@ GGZDBResult _ggzdb_player_add(ggzdbPlayerEntry *pe)
 	email_quoted = _ggz_sql_escape(pe->email);
 
 	snprintf(query, sizeof(query), "INSERT INTO users "
-		 "(handle, password, name, email, lastlogin, permissions) "
-		 "VALUES ('%s', '%s', '%s', '%s', %li, %u)",
+		 "(handle, password, name, email, lastlogin, permissions, firstlogin) "
+		 "VALUES ('%s', '%s', '%s', '%s', %li, %u, %li)",
 		 handle_quoted, password_quoted, name_quoted, email_quoted,
-		 pe->last_login, pe->perms);
+		 pe->last_login, pe->perms, time(NULL));
 
 	if (handle_quoted)
 		ggz_free(handle_quoted);
