@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/11/99
  * Desc: Datatypes used by server
- * $Id: datatypes.h 7861 2006-02-13 07:02:50Z josef $
+ * $Id: datatypes.h 8611 2006-10-03 03:27:59Z jdorje $
  *
  * Copyright (C) 1999-2002 Brent Hendricks.
  *
@@ -48,8 +48,7 @@ typedef struct {
 	int stat_clr_time;
 	int main_port;
 	const char *interface;
-	const char *game_dir;
-	const char *tmp_dir;
+	const char *game_exec_dir;
 	const char *conf_dir;
 	const char *data_dir;
 	const char *motd_file;
@@ -61,18 +60,23 @@ typedef struct {
 	ggztime_t ping_freq;
 	ggztime_t lag_class_time[4];
 	ggztime_t room_update_freq;
+	const char *dbtype;
 	const char *dbhost;
 	const char *dbname;
 	const char *dbusername;
 	const char *dbpassword;
 	const char *dbhashing;
+	const char *dbhashencoding;
 	bool tls_use;
 	const char *tls_key;
 	const char *tls_cert;
 	const char *tls_password;
 	const char *dump_file;
 	bool announce_lan;
-	bool announce_metaserver;
+	const char *announce_metaserver;
+	const char *metausername;
+	const char *metapassword;
+	bool reconfigure_rooms;
 } Options;
 
 
@@ -91,6 +95,7 @@ typedef struct GameInfo {
 	pthread_rwlock_t lock; /* This lock is used though not needed. */
 	char name[MAX_GAME_NAME_LEN + 1];    /* Game name */
 	char *version; /* Game version; cleanup() */
+	char *game; /* Internal game name as per its .dsc file; cleanup() */
 	
 	/* Protocol data - the protocol engine is the name of the
 	   protocol, and the version is the protocol version.  These
