@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 9/22/01
  * Desc: Functions for handling network IO
- * $Id: net.c 8765 2006-12-27 11:20:31Z josef $
+ * $Id: net.c 9722 2008-02-14 19:23:28Z josef $
  * 
  * Code for parsing XML streamed from the server
  *
@@ -235,8 +235,8 @@ GGZReturn net_set_dump_file(GGZNetIO *net, const char* filename)
 		return GGZ_OK;
 	}
 	
-	if (strcmp(filename, "stderr") == 0)
-		net->dump_file = STDERR_FILENO;
+	if (strcasecmp(filename, "stderr") == 0)
+		net->dump_file = opt.foreground ? STDERR_FILENO : -1;
 	else
 		net->dump_file = open(filename,
 				      O_WRONLY|O_CREAT|O_TRUNC, S_IRWXU);
@@ -300,7 +300,7 @@ void net_free(GGZNetIO *net)
 
 GGZReturn net_send_serverid(GGZNetIO *net, const char *srv_name, bool use_tls)
 {
-	char *xml_srv_name = ggz_xml_escape(srv_name);
+	const char *xml_srv_name = ggz_xml_escape(srv_name);
 	GGZReturn status;
 
 	/* This could go into a separate function, net_send_header, along
@@ -401,11 +401,18 @@ GGZReturn net_send_room_list_count(GGZNetIO *net, int count)
 GGZReturn net_send_room(GGZNetIO *net, int index,
 			RoomStruct *room, bool verbose)
 {
+	const char *roomname_quoted = ggz_xml_escape(room->name);
+	const char *roomdesc_quoted = ggz_xml_escape(room->description);
+
 	_net_send_line(net, "<ROOM ID='%d' NAME='%s' GAME='%d' PLAYERS='%d'>",
-		       index, room->name, room->game_type,
+		       index, roomname_quoted, room->game_type,
 		       room->player_count);
 	if (verbose && room->description)
-		_net_send_line(net, "<DESC>%s</DESC>", room->description);
+		_net_send_line(net, "<DESC>%s</DESC>", roomdesc_quoted);
+
+	ggz_free(roomname_quoted);
+	ggz_free(roomdesc_quoted);
+
 	return _net_send_line(net, "</ROOM>");
 }
 
@@ -440,28 +447,45 @@ GGZReturn net_send_type(GGZNetIO *net, int index,
 	char *spectators = bool_to_str(type->allow_spectators);
 	char *peers = bool_to_str(type->allow_peers);
 
+	const char *gamename_quoted = ggz_xml_escape(type->name);
+
 	_net_send_line(net, "<GAME ID='%d' NAME='%s' VERSION='%s'>",
-		       index, type->name, type->version);
+		       index, gamename_quoted, type->version);
 	_net_send_line(net, "<PROTOCOL ENGINE='%s' VERSION='%s'/>",
 		       type->p_engine, type->p_version);
 	_net_send_line(net, "<ALLOW PLAYERS='%s' BOTS='%s' SPECTATORS='%s' PEERS='%s'/>",
 		       players, bots, spectators, peers);
 
+	ggz_free(gamename_quoted);
 	ggz_free(players);
 	ggz_free(bots);
 
 	if (type->named_bots) {
 		for (i = 0; type->named_bots[i]; i++) {
+			const char *botname_quoted = ggz_xml_escape(type->named_bots[i][0]);
+			const char *botclass_quoted = ggz_xml_escape(type->named_bots[i][1]);
+
 			_net_send_line(net, "<BOT NAME='%s' CLASS='%s'/>",
-				type->named_bots[i][0],
-				type->named_bots[i][1]);
+				botname_quoted, botclass_quoted);
+
+			ggz_free(botname_quoted);
+			ggz_free(botclass_quoted);
 		}
 	}
 	
 	if (verbose) {
+		const char *author_quoted = ggz_xml_escape(type->author);
+		const char *homepage_quoted = ggz_xml_escape(type->homepage);
+		const char *desc_quoted = ggz_xml_escape(type->desc);
+
 		_net_send_line(net, "<ABOUT AUTHOR='%s' URL='%s'/>",
-			       type->author, type->homepage);
-		_net_send_line(net, "<DESC>%s</DESC>", type->desc);
+			author_quoted, homepage_quoted);
+		_net_send_line(net, "<DESC>%s</DESC>",
+			desc_quoted);
+
+		ggz_free(desc_quoted);
+		ggz_free(author_quoted);
+		ggz_free(homepage_quoted);
 	}
 	return _net_send_line(net, "</GAME>");
 }
@@ -527,7 +551,7 @@ GGZReturn net_send_player(GGZNetIO *net, GGZPlayer *player)
 	GGZPlayerType type = player_get_type(player);
 	const char *type_desc = ggz_playertype_to_string(type);
 	char stats[512];
-	char *player_name_quoted;
+	const char *player_name_quoted;
 	GGZReturn ret;
 
 	_net_get_player_stats_string(player, stats, sizeof(stats));
@@ -550,7 +574,7 @@ GGZReturn net_send_player(GGZNetIO *net, GGZPlayer *player)
 
 static GGZReturn _net_send_player_lag(GGZNetIO *net, GGZPlayer *player)
 {
-	char *player_name_quoted;
+	const char *player_name_quoted;
 	GGZReturn ret;
 
 	player_name_quoted = ggz_xml_escape(player->name);
@@ -570,7 +594,7 @@ static GGZReturn _net_send_player_perms(GGZNetIO *net, GGZPlayer *player)
 {
 	GGZPlayerType type = player_get_type(player);
 	const char *type_desc = ggz_playertype_to_string(type);
-	char *player_name_quoted;
+	const char *player_name_quoted;
 	GGZReturn ret;
 
 	player_name_quoted = ggz_xml_escape(player->name);
@@ -589,7 +613,7 @@ static GGZReturn _net_send_player_perms(GGZNetIO *net, GGZPlayer *player)
 static GGZReturn _net_send_player_stats(GGZNetIO *net, GGZPlayer *player)
 {
 	char stats[512];
-	char *player_name_quoted;
+	const char *player_name_quoted;
 	GGZReturn ret;
 
 	_net_get_player_stats_string(player, stats, sizeof(stats));
@@ -630,7 +654,7 @@ GGZReturn net_send_table_list_count(GGZNetIO *net, int room_id, int count)
 GGZReturn net_send_table(GGZNetIO *net, GGZTable *table)
 {
 	int i;
-	char *description_quoted;
+	const char *description_quoted;
 
 	_net_send_line(net, "<TABLE ID='%d' GAME='%d' STATUS='%d' SEATS='%d'>",
 		       table->index, table->type, table->state, 
@@ -677,7 +701,7 @@ GGZReturn net_send_chat(GGZNetIO *net, GGZChatType type,
 			const char *sender, const char *msg)
 {
 	const char *type_str = ggz_chattype_to_string(type);
-	char *sender_quoted;
+	const char *sender_quoted;
 	GGZReturn ret;
 
 	if (type == GGZ_CHAT_BEEP) {
@@ -697,7 +721,7 @@ GGZReturn net_send_chat(GGZNetIO *net, GGZChatType type,
 	sender_quoted = ggz_xml_escape(sender);
 
 	if (msg) {
-		char *msg_quoted = ggz_xml_escape(msg);
+		const char *msg_quoted = ggz_xml_escape(msg);
 		ret = _net_send_line(net, "<CHAT TYPE='%s' FROM='%s'>"
 				      "%s</CHAT>", 
 				      type_str, sender_quoted, msg_quoted);
@@ -780,7 +804,7 @@ GGZReturn net_send_player_update(GGZNetIO *net, GGZPlayerUpdateType opcode,
 {
 	GGZPlayer *player;
 	GGZPlayer p2;
-	char *name_quoted;
+	const char *name_quoted;
 	
 	switch (opcode) {
 	case GGZ_PLAYER_UPDATE_DELETE:
@@ -835,7 +859,7 @@ GGZReturn net_send_player_update(GGZNetIO *net, GGZPlayerUpdateType opcode,
 		pthread_rwlock_unlock(&player->lock);
 		_net_send_line(net, "<UPDATE TYPE='player' ACTION='stats' "
 			       "ROOM='%d'>", room_id);
-		_net_send_player_stats(net, player);
+		_net_send_player_stats(net, &p2);
 		return _net_send_line(net, "</UPDATE>");
 	case GGZ_PLAYER_UPDATE_PERMS:
 		player = hash_player_lookup(name);
@@ -1062,7 +1086,7 @@ GGZPlayerHandlerStatus net_read_data(GGZNetIO *net)
 			return GGZ_REQ_OK;
 		}
 
-		dbg_msg(GGZ_DBG_CONNECTION, "Network error reading data");
+		dbg_msg(GGZ_DBG_CONNECTION, "Network error reading data (%d)", errno);
 		return GGZ_REQ_DISCONNECT;
 	}
 
@@ -2183,7 +2207,7 @@ static GGZReturn _net_send_table_spectator(GGZNetIO *net, GGZTable *table,
 
 static GGZReturn _net_send_table_desc(GGZNetIO *net, GGZTable *table)
 {
-	char *description_quoted;
+	const char *description_quoted;
 
 	_net_send_line(net, "<TABLE ID='%d'>", table->index);
 	description_quoted = ggz_xml_escape(table->desc);
@@ -2223,13 +2247,13 @@ static GGZReturn _net_send_seat(GGZNetIO *net, GGZTableSeat *seat)
 		name = seat->name;
 	
 	if (name) {
-		const char *name_unquoted = ggz_xml_escape(name);
+		const char *name_quoted = ggz_xml_escape(name);
 
 		ret = _net_send_line(net,
 				      "<SEAT NUM='%d' TYPE='%s'>%s</SEAT>",
-				      seat->index, type_str, name_unquoted);
+				      seat->index, type_str, name_quoted);
 
-		ggz_free(name_unquoted);
+		ggz_free(name_quoted);
 		return ret;
 	} else
 		return _net_send_line(net, "<SEAT NUM='%d' TYPE='%s'/>", 
@@ -2240,8 +2264,14 @@ static GGZReturn _net_send_seat(GGZNetIO *net, GGZTableSeat *seat)
 static GGZReturn _net_send_spectator(GGZNetIO *net,
 				     GGZTableSpectator *spectator)
 {
-	return _net_send_line(net, "<SPECTATOR NUM='%d'>%s</SPECTATOR>",
-			      spectator->index, spectator->name);
+	const char *name_quoted = ggz_xml_escape(spectator->name);
+	GGZReturn ret;
+
+	ret = _net_send_line(net, "<SPECTATOR NUM='%d'>%s</SPECTATOR>",
+		spectator->index, name_quoted);
+	ggz_free(name_quoted);
+
+	return ret;
 }
 
 static GGZReturn _net_send_result(GGZNetIO *net, const char *action,
@@ -2275,8 +2305,11 @@ static GGZReturn _net_send_login_new_status(GGZNetIO *net,
 		       ggz_error_to_string(status));
 
 	/* Try to send checksum if successful */
-	if (status == E_OK)
-		_net_send_line(net, "<PASSWORD>%s</PASSWORD>", password);
+	if (status == E_OK) {
+		const char *password_quoted = ggz_xml_escape(password);
+		_net_send_line(net, "<PASSWORD>%s</PASSWORD>", password_quoted);
+		ggz_free(password_quoted);
+	}
 	
 	return _net_send_line(net, "</RESULT>");
 }
@@ -2301,8 +2334,18 @@ GGZReturn net_send_info_list_begin(GGZNetIO *net)
 GGZReturn net_send_info(GGZNetIO *net, int num, const char *realname,
 			const char *photo, const char *host)
 {
+	GGZReturn ret;
+	const char *realname_quoted = ggz_xml_escape(realname);
+	const char *photo_quoted = ggz_xml_escape(photo);
+
 	return _net_send_line(net, "<PLAYERINFO SEAT='%d' REALNAME='%s' PHOTO='%s' HOST='%s'/>",
-		num, (realname ? realname : ""), (photo ? photo : ""), (host ? host : ""));
+		num, (realname_quoted ? realname_quoted : ""),
+		(photo_quoted ? photo_quoted : ""), (host ? host : ""));
+
+	ggz_free(realname_quoted);
+	ggz_free(photo_quoted);
+
+	return ret;
 }
 
 GGZReturn net_send_info_list_end(GGZNetIO *net)
