@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 9/23/01
  * Desc: Functions for dealing with user permissions
- * $Id: perms.c 5923 2004-02-14 21:12:29Z jdorje $
+ * $Id: perms.c 8763 2006-12-27 10:02:33Z jdorje $
  *
  * Copyright (C) 2001 Brent Hendricks.
  *
@@ -27,36 +27,49 @@
 # include <config.h>		/* Site specific config */
 #endif
 
+#include "err_func.h"
 #include "ggzd.h"
 #include "ggzdb.h"
 #include "perms.h"
 #include "players.h"
 
-void perms_init(GGZPlayer *player, ggzdbPlayerEntry *pe)
+void perms_init_from_db(GGZPermset *perms, ggzdbPlayerEntry *pe)
 {
-	player->perms = pe->perms;
+	*perms = pe->perms;
+}
+
+/* Returns TRUE on success, FALSE on failure. */
+bool perms_set_to_db(GGZPermset perms, ggzdbPlayerEntry *pe)
+{
+	pe->perms = perms;
+	return (ggzdb_player_update(pe) != GGZDB_NO_ERROR);
 }
 
 
-bool perms_check(GGZPlayer *player, unsigned int perm)
+bool perms_check(GGZPlayer *player, GGZPerm perm)
 {
 	bool result;
 
+	if (perm < 0 || perm >= GGZ_PERM_COUNT) {
+	  err_msg("Invalid perm %d.", perm);
+	  return false;
+	}
+
 	pthread_rwlock_rdlock(&player->lock);
-	result = ((player->perms & perm) == perm);
+	result = ggz_perms_is_set(player->perms, perm);
 	pthread_rwlock_unlock(&player->lock);
 
 	return result;
 }
 
+GGZPerm perms_default_anon[] = {
+  GGZ_PERM_JOIN_TABLE, GGZ_PERM_LAUNCH_TABLE, GGZ_PERM_NO_STATS
+};
 
-bool perms_is_admin(GGZPlayer *player)
-{
-	return ((player->perms & PERMS_ADMIN_MASK) != 0);
-}
+/* Permissions for newly-registered user. */
+GGZPerm perms_default[] = {
+  GGZ_PERM_JOIN_TABLE, GGZ_PERM_LAUNCH_TABLE, GGZ_PERM_ROOMS_LOGIN
+};
 
-
-bool perms_is_bot(GGZPlayer *player)
-{
-	return ((player->perms & PERMS_CHAT_BOT) != 0);
-}
+const int num_perms_default_anon = ARRAY_SIZE(perms_default_anon);
+const int num_perms_default = ARRAY_SIZE(perms_default);
