@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/18/99
  * Desc: Functions for handling players
- * $Id: players.c 7879 2006-02-27 12:16:44Z josef $
+ * $Id: players.c 8801 2007-01-02 20:52:54Z jdorje $
  *
  * Desc: Functions for handling players.  These functions are all
  * called by the player handler thread.  Since this thread is the only
@@ -103,7 +103,8 @@ GGZPlayer* player_new(GGZClient *client)
 	player->transit = false;
 	player->room = -1;
 	player->login_status = GGZ_LOGIN_NONE;
-	player->perms = PERMS_DEFAULT_ANON;
+	ggz_perms_init_from_list(&player->perms,
+			     perms_default_anon, num_perms_default_anon);
 	strcpy(player->name, "<none>");
 	player->room_events = NULL;
 	player->my_events_head = NULL;
@@ -126,6 +127,8 @@ GGZPlayer* player_new(GGZClient *client)
 	player->rating = -1;
 	player->highscore = -1;
 
+	player->gagged = false;
+
 	return player;
 }
 
@@ -285,14 +288,19 @@ GGZPlayerType player_get_type(GGZPlayer *player)
 	   one now wouldn't do much good anyway, since the read lock is
 	   needed to protect the pointer itself... */
 
-	if(player->login_status == GGZ_LOGIN_ANON)
+	if (player->login_status == GGZ_LOGIN_ANON) {
 		type = GGZ_PLAYER_GUEST;
-	else if(perms_is_admin(player))
-		type = GGZ_PLAYER_ADMIN;
-	else if(perms_is_bot(player))
+	} else if (ggz_perms_is_bot(player->perms)) {
+		/* Check for a bot first, because bots may have admin
+		 * capabilities as well */
 		type = GGZ_PLAYER_BOT;
-	else
+	} else if (ggz_perms_is_admin(player->perms)) {
+		type = GGZ_PLAYER_ADMIN;
+	} else if (ggz_perms_is_host(player->perms)) {
+		type = GGZ_PLAYER_HOST;
+	} else {
 		type = GGZ_PLAYER_NORMAL;
+	}
 
 	return type;
 }
@@ -323,7 +331,7 @@ GGZPlayerHandlerStatus player_table_launch(GGZPlayer* player, GGZTable *table)
 	dbg_msg(GGZ_DBG_TABLE, "Handling table launch for %s", player->name);
 
 	/* Check permissions */
-	if(perms_check(player, PERMS_LAUNCH_TABLE) == PERMS_DENY) {
+	if (!perms_check(player, GGZ_PERM_LAUNCH_TABLE)) {
 		dbg_msg(GGZ_DBG_TABLE, "%s insufficient perms to launch",
 			player->name);
 		if(net_send_table_launch(player->client->net,
@@ -438,7 +446,7 @@ static GGZTable *check_table_perms(GGZPlayer *player,
 
 	/* perms_check does a read-lock on the player.  This will succeed
 	 * since we are the player thread. */
-	if (perms_check(player, PERMS_EDIT_TABLES)) {
+	if (perms_check(player, GGZ_PERM_EDIT_TABLES)) {
 		return table;
 	}
 
@@ -481,13 +489,17 @@ GGZPlayerHandlerStatus player_table_desc_update(GGZPlayer* player,
 	return GGZ_REQ_OK;
 }
 
+/* Called by the network code when the client sends an <UPDATE> for the
+   table seat.  This can be used for instance to toggle bot and open
+   seats. */
 GGZPlayerHandlerStatus player_table_seat_update(GGZPlayer *player,
 						int room_id, int table_id,
 						GGZTableSeat *seat)
 {
 	bool allow;
-
 	GGZTable *table = check_table_perms(player, room_id, table_id);
+	GGZNumberList bots_allowed;
+
 	if (!table) {
 		if(net_send_update_result(player->client->net,
 					  E_NO_PERMISSION) < 0)
@@ -497,9 +509,23 @@ GGZPlayerHandlerStatus player_table_seat_update(GGZPlayer *player,
 
 	pthread_rwlock_rdlock(&game_types[table->type].lock);
 	allow = game_types[table->type].allow_leave;
+	bots_allowed = game_types[table->type].bot_allow_list;
 	pthread_rwlock_unlock(&game_types[table->type].lock);
 	pthread_rwlock_unlock(&table->lock);
 
+	/* Check if adding a bot player is allowed. */
+	if (allow && seat->type == GGZ_SEAT_BOT) {
+		const int bots = seats_count(table, GGZ_SEAT_BOT);
+
+		if (!ggz_numberlist_isset(&bots_allowed, bots + 1)) {
+			allow = false;
+		}
+	}
+
+	/* FIXME: I think this may be a significiant security hole, as we do
+	   not do full checking of the seat change here and it is not
+	   checked at all later that I can see. --jdorje */
+
 	if (!allow) {
 		if(net_send_update_result(player->client->net,
 					  E_LEAVE_FORBIDDEN) < 0)
@@ -619,7 +645,7 @@ GGZPlayerHandlerStatus player_table_join(GGZPlayer* player,
 		status = E_AT_TABLE;
 	else if (player->transit)
 		status = E_IN_TRANSIT;
-	else if (perms_check(player, PERMS_JOIN_TABLE) == PERMS_DENY)
+	else if (!perms_check(player, GGZ_PERM_JOIN_TABLE))
 		status = E_NO_PERMISSION;
 	else if (player->game_fd < 0) {
 		/* You have to establish the channel (direct connection)
@@ -655,7 +681,7 @@ GGZPlayerHandlerStatus player_table_join_spectator(GGZPlayer* player, int index)
 		status = E_AT_TABLE;
 	else if (player->transit)
 		status = E_IN_TRANSIT;
-	else if (perms_check(player, PERMS_JOIN_TABLE) == PERMS_DENY)
+	else if (!perms_check(player, GGZ_PERM_JOIN_TABLE))
 		status = E_NO_PERMISSION;
 	else /* Send a join event to the table */
 		status = player_transit(player, GGZ_TRANSIT_JOIN_SPECTATOR,
@@ -983,9 +1009,12 @@ GGZPlayerHandlerStatus player_table_info(GGZPlayer *player, int seat_num)
 
 			}
 
-			ggz_free(realname);
-			ggz_free(photo);
-			ggz_free(host);
+			if(realname)
+				ggz_free(realname);
+			if(photo)
+				ggz_free(photo);
+			if(host)
+				ggz_free(host);
 		}
 	}
 	
@@ -1355,6 +1384,7 @@ GGZPlayerHandlerStatus player_chat(GGZPlayer* player, GGZChatType type,
 			player->name);
 		if (net_send_chat_result(player->client->net, E_NOT_IN_ROOM) < 0)
 			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_OK;
 	}
 	
 	/* Parse type */
@@ -1394,6 +1424,159 @@ GGZPlayerHandlerStatus player_chat(GGZPlayer* player, GGZChatType type,
 }
 
 
+GGZPlayerHandlerStatus player_room_admin(GGZPlayer* player, GGZAdminType type,
+					 const char *target,
+					 const char *reason)
+{
+	GGZClientReqError status = E_BAD_OPTIONS;
+	GGZPlayer* rcvr;
+
+	dbg_msg(GGZ_DBG_CHAT, "Handling admin action for %s", player->name);
+
+	/* Verify that we are in a regular room */
+	/* No lock needed: no one can change our room but us */
+	if (player->room == -1) {
+		dbg_msg(GGZ_DBG_CHAT, "%s is not in a room to perform admin action", 
+			player->name);
+		if (net_send_admin_result(player->client->net, E_NOT_IN_ROOM) < 0)
+			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_OK;
+	}
+
+	/* Only hosts and admins can do admin actions */
+	if (!ggz_perms_is_host(player->perms)) {
+		dbg_msg(GGZ_DBG_CHAT, "%s is no host!", player->name);
+		if (net_send_admin_result(player->client->net, E_NO_PERMISSION) < 0)
+			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_OK;
+	}
+
+	/* Find out target player. Returns with write-lock on. */
+	rcvr = hash_player_lookup(target);
+
+	/* Parse type */
+	switch (type) {
+	case GGZ_ADMIN_UNKNOWN:
+		/* Do nothing. Better safe than sorry. */
+		break;
+	case GGZ_ADMIN_GAG:
+		dbg_msg(GGZ_DBG_CHAT, "%s gags %s", player->name, target);
+		if (rcvr == NULL || rcvr == player) {
+			status = E_USR_LOOKUP;
+		} else {
+			rcvr->gagged = true;
+			status = E_OK;
+		}
+		break;
+	case GGZ_ADMIN_UNGAG:
+		dbg_msg(GGZ_DBG_CHAT, "%s ungags %s again", player->name, target);
+		if (rcvr == NULL || rcvr == player) {
+			status = E_USR_LOOKUP;
+		} else {
+			rcvr->gagged = false;
+			status = E_OK;
+		}
+		break;
+	case GGZ_ADMIN_KICK:
+		dbg_msg(GGZ_DBG_CHAT, "%s kicks %s, reason: %s", player->name, target, reason);
+		if (rcvr == NULL || rcvr == player) {
+			status = E_USR_LOOKUP;
+		} else if (reason == NULL) {
+			status = E_BAD_OPTIONS;
+		} else {
+			/* FIXME 1: session_over should be bool, not char */
+			/* FIXME: We temporarily take away the wr-lock since player_enqueue needs it */
+			pthread_rwlock_unlock(&rcvr->lock);	
+			chat_player_enqueue(target, GGZ_CHAT_PERSONAL, player, reason);
+
+			/* Recover the rcvr player - a new lookup is needed */
+			rcvr = hash_player_lookup(target);
+			if (rcvr) rcvr->client->session_over = 1;
+			status = E_OK;
+		}
+		break;
+	case GGZ_ADMIN_BAN:
+		dbg_msg(GGZ_DBG_CHAT, "%s bans %s", player->name, target);
+		/* This is not implemented/used yet. */
+		break;
+	}
+
+	/* Now we must return write-lock! */
+	if (rcvr) {
+		pthread_rwlock_unlock(&rcvr->lock);
+	}
+
+	dbg_msg(GGZ_DBG_CHAT, "%s's admin action result: %d", player->name, status);
+	if (net_send_admin_result(player->client->net, status) < 0)
+		return GGZ_REQ_DISCONNECT;
+
+	/* Don't return the admin error code */
+	return GGZ_REQ_OK;
+}
+
+
+GGZPlayerHandlerStatus player_perms_admin(GGZPlayer *player,
+					  const char *target,
+					  GGZPerm perm, bool set)
+{
+	GGZPlayer* rcvr;
+
+	dbg_msg(GGZ_DBG_CHAT, "Handling admin perm action for %s",
+		player->name);
+
+	/* Only admins can do permission actions */
+	if (!ggz_perms_is_admin(player->perms)) {
+		dbg_msg(GGZ_DBG_CHAT, "%s is no admin!", player->name);
+		if (net_send_admin_result(player->client->net,
+					  E_NO_PERMISSION) < 0)
+			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_OK;
+	}
+
+	/* Find out target player. Returns with write-lock on. */
+	rcvr = hash_player_lookup(target);
+	if (rcvr == NULL) {
+		if (net_send_admin_result(player->client->net,
+					  E_USR_LOOKUP) < 0)
+			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_OK;
+	}
+	if (rcvr == player || rcvr->login_status == GGZ_LOGIN_ANON) {
+		pthread_rwlock_unlock(&rcvr->lock);
+		if (net_send_admin_result(player->client->net,
+					  E_BAD_OPTIONS) < 0)
+			return GGZ_REQ_DISCONNECT;
+		return GGZ_REQ_OK;
+	}
+
+	if (ggz_perms_is_set(rcvr->perms, perm) != set) {
+		ggzdbPlayerEntry entry;
+
+		snprintf(entry.handle, sizeof(entry.handle), rcvr->name);
+		if (ggzdb_player_get(&entry) != GGZDB_NO_ERROR) {
+			pthread_rwlock_unlock(&rcvr->lock);
+			if (net_send_admin_result(player->client->net,
+						  E_USR_LOOKUP) < 0)
+				return GGZ_REQ_DISCONNECT;
+			return GGZ_REQ_OK;
+		}
+
+		ggz_perms_set(&rcvr->perms, perm, set);
+		perms_set_to_db(rcvr->perms, &entry);
+
+		room_update_event(rcvr->name, GGZ_PLAYER_UPDATE_PERMS,
+				  rcvr->room, -1);
+	}
+
+	pthread_rwlock_unlock(&rcvr->lock);
+	if (net_send_admin_result(player->client->net, E_OK) < 0)
+		return GGZ_REQ_DISCONNECT;
+
+	/* Don't return the admin error code */
+	return GGZ_REQ_OK;
+}
+
+
 GGZPlayerHandlerStatus player_motd(GGZPlayer* player)
 {
 	dbg_msg(GGZ_DBG_CHAT, "Handling motd request for %s", player->name);
@@ -1428,10 +1611,14 @@ GGZPlayerHandlerStatus player_send_room_update(GGZPlayer *player)
 		return GGZ_REQ_OK;
 	}
 
-	for (i = 0; i < room_info.num_rooms; i++) {
+	for (i = 0; i < room_get_num_rooms(); i++) {
 		int player_count;
 		time_t last_player_change;
 
+		if (room_is_removed(i)) {
+			continue;
+		}
+
 		pthread_rwlock_rdlock(&rooms[i].lock);
 		player_count = rooms[i].player_count;
 		last_player_change = rooms[i].last_player_change;
@@ -1440,7 +1627,7 @@ GGZPlayerHandlerStatus player_send_room_update(GGZPlayer *player)
 		if (curr_time - last_player_change <= opt.room_update_freq) {
 			net_send_room_update(player->client->net,
 					     GGZ_ROOM_UPDATE_PLAYER_COUNT, i,
-					     player_count);
+					     player_count, NULL);
 		}
 	}
 
