@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 10/18/99
  * Desc: Functions for handling players
- * $Id: players.h 7424 2005-08-15 09:00:27Z josef $
+ * $Id: players.h 8763 2006-12-27 10:02:33Z jdorje $
  *
  * Copyright (C) 1999,2000 Brent Hendricks.
  *
@@ -116,6 +116,9 @@ struct _GGZPlayer {
 	/* Player permissions settings */
 	unsigned int perms;
 	/*list_t *op_rooms;	Not yet */
+
+	/* Gagging. A gagged player's chat is not relayed to others. */
+	bool gagged;
 };
 
 
@@ -129,6 +132,12 @@ GGZEventFuncReturn player_launch_callback(void* target, size_t size,
 
 GGZPlayerHandlerStatus player_chat(GGZPlayer* player, GGZChatType type,
 				   const char *target, const char *msg);
+GGZPlayerHandlerStatus player_room_admin(GGZPlayer* player, GGZAdminType type,
+					 const char *target,
+					 const char *reason);
+GGZPlayerHandlerStatus player_perms_admin(GGZPlayer *player,
+					  const char *target,
+					  GGZPerm perm, bool set);
 GGZPlayerHandlerStatus player_table_launch(GGZPlayer* player, GGZTable *table);
 GGZPlayerHandlerStatus player_table_join(GGZPlayer* player,
 					 int table_index,
