@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 1/9/00
  * Desc: Functions for handling tables
- * $Id: table.c 8776 2006-12-30 10:14:45Z oojah $
+ * $Id: table.c 9718 2008-02-14 18:04:30Z josef $
  *
  * Copyright (C) 1999-2002 Brent Hendricks.
  *
@@ -246,7 +246,7 @@ static GGZReturn table_check(GGZTable* table)
 	dbg_msg(GGZ_DBG_TABLE, "Open Seats : %d", seats_count(table, GGZ_SEAT_OPEN));
 	dbg_msg(GGZ_DBG_TABLE, "Resv.Seats : %d", seats_count(table, GGZ_SEAT_RESERVED));
 	dbg_msg(GGZ_DBG_TABLE, "State      : %d", table->state);
-	dbg_msg(GGZ_DBG_TABLE, "GGZdMod    : %x", (unsigned int)table->ggzdmod);
+	dbg_msg(GGZ_DBG_TABLE, "GGZdMod    : %p", table->ggzdmod);
 	
 	/* FIXME: this correctly logs everything about the GGZdMod object, but it
 	   will incorrectly be labeled as coming from the table itself instead of
