@@ -4,7 +4,7 @@
  * Project: GGZ Server
  * Date: 3/26/00
  * Desc: Functions for handling table transits
- * $Id: transit.c 7107 2005-04-15 17:54:31Z jdorje $
+ * $Id: transit.c 8887 2007-01-10 01:27:41Z jdorje $
  *
  * Copyright (C) 2000 Brent Hendricks.
  *
@@ -160,7 +160,7 @@ static GGZEventFuncReturn transit_seat_event_callback(void* target,
 		return GGZ_EVENT_OK;
 	}
 
-	if (seat->type == GGZ_SEAT_PLAYER && !spectating) {
+	if (!spectating && seat->type == GGZ_SEAT_PLAYER) {
 		int index = transit_find_required_seat(table, event->caller);
 
 		if (index >= 0) {
@@ -170,8 +170,8 @@ static GGZEventFuncReturn transit_seat_event_callback(void* target,
 	}
 
 	/* Try to find a seat if one isn't specified */
-	if ((seat->type == GGZ_SEAT_PLAYER || spectating)
-	    && seat->index == GGZ_SEATNUM_ANY) {
+	if (seat->index == GGZ_SEATNUM_ANY
+	    && (spectating || seat->type == GGZ_SEAT_PLAYER)) {
 		if (spectating)
 			seat->index = transit_find_available_spectator(table,
 						event->caller);
@@ -203,7 +203,11 @@ static GGZEventFuncReturn transit_seat_event_callback(void* target,
 		transit_player_event(event->caller, event->transit,
 				     E_SEAT_ASSIGN_FAIL,
 				     event->caller, -1, 0);
-		return GGZ_EVENT_ERROR;
+
+		/* This isn't an error really; it can happen if two joins
+		   happen simultaneously.  If we returned an error here it
+		   would cause the table to be killed. */
+		return GGZ_EVENT_OK;
 	}
 
 	return GGZ_EVENT_OK;
