@@ -4,7 +4,7 @@
  * Project: ggzdmod
  * Date: 11/18/01
  * Desc: Functions for reading/writing messages from/to game modules
- * $Id: io.h 7887 2006-03-07 09:56:51Z josef $
+ * $Id: io.h 8001 2006-04-24 07:17:07Z josef $
  *
  * This file contains the backend for the ggzdmod library.  This
  * library facilitates the communication between the GGZ server (ggzd)
@@ -33,7 +33,7 @@
 #include "mod.h"
 
 /* Functions for sending IO messages */
-int _io_send_launch(int fd, int seats, int spectators);
+int _io_send_launch(int fd, const char *game, int seats, int spectators);
 int _io_send_seat_change(int fd, const GGZSeat *seat);
 int _io_send_spectator_change(int fd, const GGZSpectator *spectator);
 int _io_send_state(int fd, GGZdModState state);
